import { __extends } from "tslib";
/**
 * An error thrown if parsing failed. Custom checkers and transformers can throw this error to notify that the operation
 * has failed.
 */
var ValidationError = /** @class */ (function (_super) {
    __extends(ValidationError, _super);
    /**
     * Creates a new {@linkcode ValidationError} instance.
     *
     * @param issues The mutable list of partially defined issues that have caused an error.
     */
    function ValidationError(issues) {
        var _newTarget = this.constructor;
        var _this = this;
        var message = '';
        for (var _i = 0, issues_1 = issues; _i < issues_1.length; _i++) {
            var issue = issues_1[_i];
            inflateIssue(issue);
            message +=
                '\n' + issue.code + ' at /' + issue.path.join('/') + (issue.message != null ? ': ' + issue.message : '');
        }
        _this = _super.call(this, message) || this;
        Object.setPrototypeOf(_this, _newTarget.prototype);
        _this.name = 'ValidationError';
        _this.issues = issues;
        return _this;
    }
    return ValidationError;
}(Error));
export { ValidationError };
export function inflateIssue(issue) {
    if (issue.code == null) {
        issue.code = 'unknown';
    }
    if (issue.path == null) {
        issue.path = [];
    }
}
