import { AnyShape, ArrayShape } from '../shapes';
import { Message, TypeConstraintOptions } from '../shared-types';
/**
 * Creates the tuple shape.
 *
 * @param shapes The list of tuple element shapes.
 * @param options The constraint options or an issue message.
 * @template U The tuple elements.
 */
export declare function tuple<U extends readonly [AnyShape, ...AnyShape[]]>(shapes: U, options?: TypeConstraintOptions | Message): ArrayShape<U, null>;
/**
 * Creates the tuple shape with rest elements.
 *
 * @param shapes The list of tuple element shapes.
 * @param restShape The shape of rest elements.
 * @param options The constraint options or an issue message.
 * @template U The head tuple elements.
 * @template R The rest tuple elements.
 */
export declare function tuple<U extends readonly [AnyShape, ...AnyShape[]], R extends AnyShape | null = null>(shapes: U, restShape?: R | null, options?: TypeConstraintOptions | Message): ArrayShape<U, R>;
