import { __extends } from "tslib";
import { Shape } from './Shape';
import { booleanTypes, createIssueFactory } from '../utils';
import { CODE_TYPE, MESSAGE_BOOLEAN_TYPE, TYPE_BOOLEAN } from '../constants';
/**
 * The shape of the bigint value.
 */
var BooleanShape = /** @class */ (function (_super) {
    __extends(BooleanShape, _super);
    /**
     * Creates a new {@linkcode BooleanShape} instance.
     *
     * @param options The type constraint options or the type issue message.
     */
    function BooleanShape(options) {
        var _this = _super.call(this, booleanTypes) || this;
        _this._issueFactory = createIssueFactory(CODE_TYPE, MESSAGE_BOOLEAN_TYPE, options, TYPE_BOOLEAN);
        return _this;
    }
    BooleanShape.prototype.apply = function (input, options) {
        var _applyChecks = this._applyChecks;
        if (typeof input !== 'boolean') {
            return [this._issueFactory(input, options)];
        }
        if (_applyChecks !== null) {
            return _applyChecks(input, null, options);
        }
        return null;
    };
    return BooleanShape;
}(Shape));
export { BooleanShape };
