import { __extends } from "tslib";
import { Shape } from './Shape';
import { createIssueFactory } from '../utils';
import { CODE_ENUM, MESSAGE_ENUM } from '../constants';
/**
 * The shape that constrains an input to one of values.
 *
 * @template T Allowed values.
 */
var EnumShape = /** @class */ (function (_super) {
    __extends(EnumShape, _super);
    /**
     * Creates a new {@linkcode EnumShape} instance.
     *
     * @param values The list of values allowed for the input.
     * @param options The type constraint options or an issue message.
     */
    function EnumShape(values, options) {
        var _this = _super.call(this, values.map(Shape.typeof)) || this;
        _this.values = values;
        _this._issueFactory = createIssueFactory(CODE_ENUM, MESSAGE_ENUM, options, values);
        return _this;
    }
    EnumShape.prototype.apply = function (input, options) {
        var _applyChecks = this._applyChecks;
        if (!this.values.includes(input)) {
            return [this._issueFactory(input, options)];
        }
        if (_applyChecks !== null) {
            return _applyChecks(input, null, options);
        }
        return null;
    };
    return EnumShape;
}(Shape));
export { EnumShape };
