import { Shape } from './Shape';
import { ApplyResult, Message, ParseOptions, TypeConstraintOptions } from '../shared-types';
/**
 * The shape of the class instance.
 *
 * @template C The class constructor.
 */
export declare class InstanceShape<C extends new (...args: any[]) => any> extends Shape<InstanceType<C>> {
    readonly ctor: C;
    protected _issueFactory: (input: unknown, options: Readonly<ParseOptions>) => import("../shared-types").Issue;
    /**
     * Creates a new {@linkcode InstanceShape} instance.
     *
     * @param ctor The class constructor.
     * @param options The type constraint options or the type issue message.
     * @template C The class constructor.
     */
    constructor(ctor: C, options?: TypeConstraintOptions | Message);
    apply(input: unknown, options: ParseOptions): ApplyResult<InstanceType<C>>;
}
