import { __extends } from "tslib";
import { Shape } from './Shape';
import { arrayTypes, createIssueFactory, objectTypes } from '../utils';
import { CODE_INSTANCE, MESSAGE_INSTANCE } from '../constants';
/**
 * The shape of the class instance.
 *
 * @template C The class constructor.
 */
var InstanceShape = /** @class */ (function (_super) {
    __extends(InstanceShape, _super);
    /**
     * Creates a new {@linkcode InstanceShape} instance.
     *
     * @param ctor The class constructor.
     * @param options The type constraint options or the type issue message.
     * @template C The class constructor.
     */
    function InstanceShape(ctor, options) {
        var _this = _super.call(this, ctor === Array || Array.prototype.isPrototypeOf(ctor.prototype) ? arrayTypes : objectTypes) || this;
        _this.ctor = ctor;
        _this._issueFactory = createIssueFactory(CODE_INSTANCE, MESSAGE_INSTANCE, options, ctor);
        return _this;
    }
    InstanceShape.prototype.apply = function (input, options) {
        var _applyChecks = this._applyChecks;
        if (!(input instanceof this.ctor)) {
            return [this._issueFactory(input, options)];
        }
        if (_applyChecks !== null) {
            return _applyChecks(input, null, options);
        }
        return null;
    };
    return InstanceShape;
}(Shape));
export { InstanceShape };
