import { AnyShape, Shape } from './Shape';
import { ApplyResult, ParseOptions } from '../shared-types';
/**
 * Lazily resolves a shape using the provider.
 *
 * @template S The base shape.
 */
export declare class LazyShape<S extends AnyShape> extends Shape<S['input'], S['output']> {
    /**
     * The base shape.
     */
    readonly shape: S;
    /**
     * The callback that returns the base shape.
     */
    protected _provider: () => S;
    /**
     * Creates a new {@linkcode LazyShape} instance.
     *
     * @param provider The provider that returns the base shape.
     * @param async If `true` the shape would support both sync and async base shapes, otherwise only sync shapes are
     * allowed.
     * @template S The base shape.
     */
    constructor(provider: () => S, async: boolean);
    apply(input: any, options: ParseOptions): ApplyResult<S['output']>;
    applyAsync(input: unknown, options: ParseOptions): Promise<ApplyResult<S['output']>>;
}
