import { __extends } from "tslib";
import { Shape } from './Shape';
import { anyTypes, isArray } from '../utils';
/**
 * Lazily resolves a shape using the provider.
 *
 * @template S The base shape.
 */
var LazyShape = /** @class */ (function (_super) {
    __extends(LazyShape, _super);
    /**
     * Creates a new {@linkcode LazyShape} instance.
     *
     * @param provider The provider that returns the base shape.
     * @param async If `true` the shape would support both sync and async base shapes, otherwise only sync shapes are
     * allowed.
     * @template S The base shape.
     */
    function LazyShape(provider, async) {
        var _this = _super.call(this, anyTypes, async) || this;
        _this._provider = provider;
        return _this;
    }
    LazyShape.prototype.apply = function (input, options) {
        var _applyChecks = this._applyChecks;
        var output = input;
        var issues;
        var result = this.shape.apply(input, options);
        if (result !== null) {
            if (isArray(result)) {
                return result;
            }
            output = result.value;
        }
        if (_applyChecks !== null) {
            issues = _applyChecks(output, null, options);
            if (issues !== null) {
                return issues;
            }
        }
        return result;
    };
    LazyShape.prototype.applyAsync = function (input, options) {
        var _applyChecks = this._applyChecks;
        return this.shape.applyAsync(input, options).then(function (result) {
            var output = input;
            var issues;
            if (result !== null) {
                if (isArray(result)) {
                    return result;
                }
                output = result.value;
            }
            if (_applyChecks !== null) {
                issues = _applyChecks(output, null, options);
                if (issues !== null) {
                    return issues;
                }
            }
            return result;
        });
    };
    return LazyShape;
}(Shape));
export { LazyShape };
Object.defineProperty(LazyShape.prototype, 'shape', {
    get: function () {
        var shape = this._provider();
        Object.defineProperty(this, 'shape', { value: shape, enumerable: true });
        return shape;
    },
});
