import { AnyShape, Shape } from './Shape';
import { ApplyResult, Message, ParseOptions, TypeConstraintOptions } from '../shared-types';
/**
 * The shape of a value wrapped in a `Promise` instance.
 *
 * @template S The shape of the resolved value.
 */
export declare class PromiseShape<S extends AnyShape> extends Shape<Promise<S['input']>, Promise<S['output']>> {
    readonly shape: S;
    protected _issueFactory: (input: unknown, options: Readonly<ParseOptions>) => import("../shared-types").Issue;
    /**
     * Creates a new {@linkcode PromiseShape} instance.
     *
     * @param shape The shape of the resolved value.
     * @param options The type constraint options or the type issue message.
     * @template S The shape of the resolved value.
     */
    constructor(shape: S, options?: TypeConstraintOptions | Message);
    apply(input: unknown, options: ParseOptions): ApplyResult<Promise<S['output']>>;
    applyAsync(input: unknown, options: ParseOptions): Promise<ApplyResult<Promise<S['output']>>>;
}
