import { __extends } from "tslib";
import { Shape } from './Shape';
import { createIssueFactory, isArray, isEqual, objectTypes, ok } from '../utils';
import { CODE_TYPE, MESSAGE_ERROR_ASYNC, MESSAGE_PROMISE_TYPE, TYPE_PROMISE } from '../constants';
/**
 * The shape of a value wrapped in a `Promise` instance.
 *
 * @template S The shape of the resolved value.
 */
var PromiseShape = /** @class */ (function (_super) {
    __extends(PromiseShape, _super);
    /**
     * Creates a new {@linkcode PromiseShape} instance.
     *
     * @param shape The shape of the resolved value.
     * @param options The type constraint options or the type issue message.
     * @template S The shape of the resolved value.
     */
    function PromiseShape(shape, options) {
        var _this = _super.call(this, objectTypes, true) || this;
        _this.shape = shape;
        _this._issueFactory = createIssueFactory(CODE_TYPE, MESSAGE_PROMISE_TYPE, options, TYPE_PROMISE);
        return _this;
    }
    PromiseShape.prototype.apply = function (input, options) {
        throw new Error(MESSAGE_ERROR_ASYNC);
    };
    PromiseShape.prototype.applyAsync = function (input, options) {
        var _this = this;
        if (!(input instanceof Promise)) {
            return Promise.resolve([this._issueFactory(input, options)]);
        }
        var _applyChecks = this._applyChecks;
        var inputValue;
        var outputValue;
        return input
            .then(function (value) {
            inputValue = outputValue = value;
            return _this.shape.applyAsync(value, options);
        })
            .then(function (result) {
            var issues;
            var output = input;
            if (result !== null) {
                if (isArray(result)) {
                    return result;
                }
                outputValue = result.value;
                if (!isEqual(inputValue, outputValue)) {
                    output = Promise.resolve(outputValue);
                }
            }
            if (_applyChecks !== null) {
                issues = _applyChecks(output, null, options);
                if (issues !== null) {
                    return issues;
                }
            }
            return ok(output);
        });
    };
    return PromiseShape;
}(Shape));
export { PromiseShape };
