import { AnyShape, Shape } from './Shape';
import { ApplyResult, Issue, Message, ParseOptions, TypeConstraintOptions } from '../shared-types';
export declare type InferRecord<K extends Shape<string, PropertyKey> | null, V extends AnyShape, C extends 'input' | 'output'> = undefined extends V[C] ? Partial<Record<K extends Shape ? K[C] : string, V[C]>> : Record<K extends Shape ? K[C] : string, V[C]>;
/**
 * The shape that describes an object with string keys and values that conform the given shape.
 *
 * @template K The key shape.
 * @template V The value shape.
 */
export declare class RecordShape<K extends Shape<string, PropertyKey> | null, V extends AnyShape> extends Shape<InferRecord<K, V, 'input'>, InferRecord<K, V, 'output'>> {
    /**
     * The key shape or `null` if keys are preserved intact.
     */
    readonly keyShape: K;
    /**
     * The value shape.
     */
    readonly valueShape: V;
    protected _issueFactory: (input: unknown, options: Readonly<ParseOptions>) => Issue;
    /**
     * Creates a new {@linkcode RecordShape} instance.
     *
     * @param keyShape The key shape.
     * @param valueShape The value shape.
     * @param options The type constraint options or an issue message.
     * @template K The key shape or `null` if keys should be preserved intact.
     * @template V The value shape.
     */
    constructor(
    /**
     * The key shape or `null` if keys are preserved intact.
     */
    keyShape: K, 
    /**
     * The value shape.
     */
    valueShape: V, options?: TypeConstraintOptions | Message);
    at(key: unknown): AnyShape | null;
    apply(input: unknown, options: ParseOptions): ApplyResult<InferRecord<K, V, 'output'>>;
    applyAsync(input: unknown, options: ParseOptions): Promise<ApplyResult<InferRecord<K, V, 'output'>>>;
}
