import { Shape } from './Shape';
import { ApplyResult, ConstraintOptions, Message, ParseOptions, TypeConstraintOptions } from '../shared-types';
/**
 * The shape that constrains the input as a string.
 */
export declare class StringShape extends Shape<string> {
    protected _issueFactory: (input: unknown, options: Readonly<ParseOptions>) => import("../shared-types").Issue;
    /**
     * Creates a new {@linkcode StringShape} instance.
     *
     * @param options The type constraint options or the type issue message.
     */
    constructor(options?: TypeConstraintOptions | Message);
    /**
     * The shortcut to apply both {@linkcode min} and {@linkcode max} constraints.
     *
     * @param length The exact length a string must have.
     * @param options The constraint options or an issue message.
     * @returns The clone of the shape.
     */
    length(length: number, options?: ConstraintOptions | Message): this;
    /**
     * Constrains the string length to be greater than or equal to the length.
     *
     * @param length The minimum string length.
     * @param options The constraint options or an issue message.
     * @returns The clone of the shape.
     */
    min(length: number, options?: ConstraintOptions | Message): this;
    /**
     * Constrains the string length to be less than or equal to the length.
     *
     * @param length The maximum string length.
     * @param options The constraint options or an issue message.
     * @returns The clone of the shape.
     */
    max(length: number, options?: ConstraintOptions | Message): this;
    /**
     * Constrains the string to match a regexp.
     *
     * @param re The regular expression that the sting must conform.
     * @param options The constraint options or an issue message.
     * @returns The clone of the shape.
     */
    regex(re: RegExp, options?: ConstraintOptions | Message): this;
    apply(input: unknown, options: ParseOptions): ApplyResult<string>;
}
