import { AnyShape, Shape, ValueType } from './Shape';
import { ApplyResult, Issue, Message, ParseOptions, TypeConstraintOptions } from '../shared-types';
export declare type InferUnion<U extends readonly AnyShape[], C extends 'input' | 'output'> = {
    [K in keyof U]: U[K] extends AnyShape ? U[K][C] : never;
}[number];
/**
 * The shape that requires an input to conform at least one of the united shapes.
 *
 * @template U The list of united shapes.
 */
export declare class UnionShape<U extends readonly AnyShape[]> extends Shape<InferUnion<U, 'input'>, InferUnion<U, 'output'>> {
    /**
     * The list of united shapes.
     */
    readonly shapes: U;
    protected _buckets: Partial<Record<ValueType, readonly AnyShape[]>> | null;
    protected _anyBucket: readonly AnyShape[] | null;
    protected _issueFactory: (input: unknown, options: Readonly<ParseOptions>, param: unknown) => Issue;
    /**
     * Creates a new {@linkcode UnionShape} instance.
     *
     * @param shapes The list of united shapes.
     * @param options The union constraint options or an issue message.
     * @template U The list of united shapes.
     */
    constructor(
    /**
     * The list of united shapes.
     */
    shapes: U, options?: TypeConstraintOptions | Message);
    at(key: unknown): AnyShape | null;
    apply(input: unknown, options: ParseOptions): ApplyResult<InferUnion<U, 'output'>>;
    applyAsync(input: unknown, options: ParseOptions): Promise<ApplyResult<InferUnion<U, 'output'>>>;
}
/**
 * Creates a mapping from the value type to an array of shapes that are applicable.
 *
 * @param shapes The list of united shapes.
 */
export declare function createUnionBuckets(shapes: readonly AnyShape[]): {
    buckets: Partial<Record<ValueType, readonly AnyShape[]>> | null;
    anyBucket: readonly AnyShape[] | null;
};
