import { ApplyChecksCallback, Check, CheckCallback, ConstraintOptions, Issue, Message, Ok, ParseOptions, ReadonlyDict, TypeConstraintOptions } from './shared-types';
import { AnyShape, Shape, ValueType } from './shapes/Shape';
export declare const objectTypes: ValueType[];
export declare const arrayTypes: ValueType[];
export declare const stringTypes: ValueType[];
export declare const numberTypes: ValueType[];
export declare const bigintTypes: ValueType[];
export declare const booleanTypes: ValueType[];
export declare const anyTypes: ValueType[];
export declare const neverTypes: ValueType[];
export declare function ok<T>(value: T): Ok<T>;
export declare function unique<T>(arr: T[]): T[];
export declare function unique<T>(arr: readonly T[]): readonly T[];
export declare const isArray: (arg: any) => arg is any[];
export declare const getPrototypeOf: (o: any) => any;
export declare function isEqual(a: unknown, b: unknown): boolean;
export declare function isObjectLike(value: unknown): value is Record<any, any>;
export declare function isPlainObject(value: unknown): value is Record<any, any>;
export declare function isAsyncShapes(shapes: readonly AnyShape[]): boolean;
/**
 * The convenient shortcut to add built-in checks to shapes.
 */
export declare function appendCheck<S extends Shape>(shape: S, key: string | undefined, options: ConstraintOptions | Message | undefined, param: unknown, cb: CheckCallback<S['output']>): S;
/**
 * Returns a function that creates a new issue.
 *
 * @param code The code of the issue.
 * @param defaultMessage The default message that is used if message isn't provided through options.
 * @param options Options provided by the user.
 * @param param The param that is added to the issue.
 * @returns The callback that takes an input and returns an issue.
 */
export declare function createIssueFactory(code: unknown, defaultMessage: unknown, options: TypeConstraintOptions | Message | undefined, param: unknown): (input: unknown, options: Readonly<ParseOptions>) => Issue;
/**
 * Returns a function that creates a new issue.
 *
 * @param code The code of the issue.
 * @param defaultMessage The default message that is used if message isn't provided through options.
 * @param options Options provided by the user.
 * @returns The callback that takes an input and a param, and returns an issue.
 */
export declare function createIssueFactory(code: unknown, defaultMessage: unknown, options: TypeConstraintOptions | Message | undefined): (input: unknown, options: Readonly<ParseOptions>, param: unknown) => Issue;
export declare function unshiftPath(issues: Issue[], key: unknown): void;
export declare function concatIssues(issues: Issue[] | null, result: Issue[]): Issue[];
export declare function pushIssue(issues: Issue[] | null, result: Issue): Issue[];
export declare function captureIssues(error: unknown): Issue[];
export declare type Flags = number[] | number;
export declare function setFlag(flags: Flags, index: number): Flags;
export declare function isFlagSet(flag: Flags, index: number): boolean;
export declare function setKeyValue(obj: Record<any, any>, key: PropertyKey, value: unknown): void;
export declare function cloneEnumerableKeys(input: ReadonlyDict, keyCount?: number): ReadonlyDict;
export declare function cloneKnownKeys(input: ReadonlyDict, keys: readonly string[]): ReadonlyDict;
export declare function createApplyChecksCallback(checks: Check[]): ApplyChecksCallback | null;
export declare function getInputTypes(shapes: readonly AnyShape[]): ValueType[];
