import { inflateIssue, ValidationError } from './ValidationError';
export var objectTypes = ['object'];
export var arrayTypes = ['array'];
export var stringTypes = ['string'];
export var numberTypes = ['number'];
export var bigintTypes = ['bigint'];
export var booleanTypes = ['boolean'];
export var anyTypes = ['any'];
export var neverTypes = ['never'];
export function ok(value) {
    return { ok: true, value: value };
}
export function unique(arr) {
    var uniqueArr = null;
    for (var i = 0; i < arr.length; ++i) {
        var value = arr[i];
        if (arr.includes(value, i + 1)) {
            if (uniqueArr === null) {
                uniqueArr = arr.slice(0, i);
            }
            continue;
        }
        if (uniqueArr !== null) {
            uniqueArr.push(value);
        }
    }
    return uniqueArr || arr;
}
export var isArray = Array.isArray;
export var getPrototypeOf = Object.getPrototypeOf;
export function isEqual(a, b) {
    return a === b || (a !== a && b !== b);
}
export function isObjectLike(value) {
    return value !== null && typeof value === 'object';
}
export function isPlainObject(value) {
    var prototype;
    return isObjectLike(value) && ((prototype = getPrototypeOf(value)) === null || prototype.constructor === Object);
}
export function isAsyncShapes(shapes) {
    var async = false;
    for (var i = 0; i < shapes.length && !async; ++i) {
        async = shapes[i].async;
    }
    return async;
}
/**
 * The convenient shortcut to add built-in checks to shapes.
 */
export function appendCheck(shape, key, options, param, cb) {
    return shape.check(cb, {
        key: key,
        unsafe: options !== null && typeof options === 'object' && options.unsafe,
        param: param,
    });
}
export function createIssueFactory(code, defaultMessage, options, param) {
    var paramKnown = arguments.length === 4;
    var meta;
    var message = defaultMessage;
    if (options !== null && typeof options === 'object') {
        if (options.message !== undefined) {
            message = options.message;
        }
        meta = options.meta;
    }
    else if (typeof options === 'function') {
        message = options;
    }
    else if (options != null) {
        message = options;
    }
    if (typeof message === 'function') {
        if (paramKnown) {
            return function (input, options) {
                return { code: code, path: [], input: input, message: message(param, code, input, meta, options), param: param, meta: meta };
            };
        }
        else {
            return function (input, options, param) {
                return { code: code, path: [], input: input, message: message(param, code, input, meta, options), param: param, meta: meta };
            };
        }
    }
    if (typeof message === 'string') {
        if (paramKnown) {
            message = message.replace('%s', param);
        }
        else if (message.indexOf('%s') !== -1) {
            return function (input, options, param) {
                return { code: code, path: [], input: input, message: message.replace('%s', param), param: param, meta: meta };
            };
        }
    }
    if (paramKnown) {
        return function (input) {
            return { code: code, path: [], input: input, message: message, param: param, meta: meta };
        };
    }
    else {
        return function (input, options, param) {
            return { code: code, path: [], input: input, message: message, param: param, meta: meta };
        };
    }
}
export function unshiftPath(issues, key) {
    var issuesLength = issues.length;
    for (var i = 0; i < issuesLength; ++i) {
        issues[i].path.unshift(key);
    }
}
export function concatIssues(issues, result) {
    if (issues !== null) {
        issues.push.apply(issues, result);
        return issues;
    }
    return result;
}
export function pushIssue(issues, result) {
    if (issues !== null) {
        issues.push(result);
        return issues;
    }
    return [result];
}
export function captureIssues(error) {
    if (error instanceof ValidationError) {
        return error.issues;
    }
    throw error;
}
export function setFlag(flags, index) {
    if (typeof flags === 'number') {
        if (index < 32) {
            return flags | (1 << index);
        }
        flags = [flags, 0, 0];
    }
    flags[index >> 5] |= 1 << index % 32;
    return flags;
}
export function isFlagSet(flag, index) {
    if (typeof flag === 'number') {
        return 0 !== flag >>> index;
    }
    else {
        return 0 !== flag[index >> 5] >>> index % 32;
    }
}
export function setKeyValue(obj, key, value) {
    if (key === '__proto__') {
        Object.defineProperty(obj, key, { value: value, writable: true, enumerable: true, configurable: true });
    }
    else {
        obj[key] = value;
    }
}
export function cloneEnumerableKeys(input, keyCount) {
    if (keyCount === void 0) { keyCount = -1; }
    var output = {};
    if (keyCount < 0) {
        for (var key in input) {
            setKeyValue(output, key, input[key]);
        }
    }
    if (keyCount > 0) {
        var index = 0;
        for (var key in input) {
            if (index === keyCount) {
                break;
            }
            setKeyValue(output, key, input[key]);
            ++index;
        }
    }
    return output;
}
export function cloneKnownKeys(input, keys) {
    var output = {};
    var keysLength = keys.length;
    for (var i = 0; i < keysLength; ++i) {
        var key = keys[i];
        if (key in input) {
            setKeyValue(output, key, input[key]);
        }
    }
    return output;
}
export function createApplyChecksCallback(checks) {
    var checksLength = checks.length;
    if (checksLength === 0) {
        return null;
    }
    if (checksLength === 1) {
        var _a = checks[0], unsafe0_1 = _a.unsafe, cb0_1 = _a.callback;
        return function (output, issues, options) {
            if (issues === null || unsafe0_1) {
                var result = void 0;
                try {
                    result = cb0_1(output, options);
                }
                catch (error) {
                    return concatIssues(issues, captureIssues(error));
                }
                if (result != null) {
                    return appendIssue(issues, result);
                }
            }
            return issues;
        };
    }
    if (checksLength === 2) {
        var _b = checks[0], unsafe0_2 = _b.unsafe, cb0_2 = _b.callback, _c = checks[1], unsafe1_1 = _c.unsafe, cb1_1 = _c.callback;
        return function (output, issues, options) {
            if (issues === null || unsafe0_2) {
                var result = void 0;
                try {
                    result = cb0_2(output, options);
                }
                catch (error) {
                    issues = concatIssues(issues, captureIssues(error));
                    if (!options.verbose) {
                        return issues;
                    }
                }
                if (result != null) {
                    issues = appendIssue(issues, result);
                    if (issues !== null && !options.verbose) {
                        return issues;
                    }
                }
            }
            if (issues === null || unsafe1_1) {
                var result = void 0;
                try {
                    result = cb1_1(output, options);
                }
                catch (error) {
                    issues = concatIssues(issues, captureIssues(error));
                }
                if (result != null) {
                    issues = appendIssue(issues, result);
                }
            }
            return issues;
        };
    }
    return function (output, issues, options) {
        for (var i = 0; i < checksLength; ++i) {
            var _a = checks[i], unsafe = _a.unsafe, callback = _a.callback;
            var result = void 0;
            if (issues !== null && !unsafe) {
                continue;
            }
            try {
                result = callback(output, options);
            }
            catch (error) {
                issues = concatIssues(issues, captureIssues(error));
                if (!options.verbose) {
                    return issues;
                }
            }
            if (result != null) {
                issues = appendIssue(issues, result);
                if (issues !== null && !options.verbose) {
                    return issues;
                }
            }
        }
        return issues;
    };
}
function appendIssue(issues, result) {
    if (isArray(result)) {
        var resultLength = result.length;
        if (resultLength === 0) {
            return issues;
        }
        for (var i = 0; i < resultLength; ++i) {
            inflateIssue(result[i]);
        }
        if (issues === null) {
            issues = result;
        }
        else {
            issues.push.apply(issues, result);
        }
    }
    else {
        inflateIssue(result);
        if (issues === null) {
            issues = [result];
        }
        else {
            issues.push(result);
        }
    }
    return issues;
}
export function getInputTypes(shapes) {
    var types = [];
    for (var _i = 0, shapes_1 = shapes; _i < shapes_1.length; _i++) {
        var shape = shapes_1[_i];
        for (var _a = 0, _b = shape.inputTypes; _a < _b.length; _a++) {
            var type = _b[_a];
            types.push(type);
        }
    }
    return types;
}
